Class: DriverSocket
- Inherits:
-
Object
- Object
- DriverSocket
- Defined in:
- lib/ibm_appconfiguration_ruby_sdk/configurations/internal/websocket_client/driver_socket.rb
Overview
Copyright 2026 IBM Corp. All Rights Reserved.
Licensed under the Apache License, Version 2.0 (the “License”); you may not use this file except in compliance with the License. You may obtain a copy of the License at
http://www.apache.org/licenses/LICENSE-2.0
Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on an “AS IS” BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the specific language governing permissions and limitations under the License.
Instance Attribute Summary collapse
-
#url ⇒ Object
readonly
Returns the value of attribute url.
Instance Method Summary collapse
-
#initialize(tcp_socket, url) ⇒ DriverSocket
constructor
A new instance of DriverSocket.
- #write(data) ⇒ Object
Constructor Details
#initialize(tcp_socket, url) ⇒ DriverSocket
Returns a new instance of DriverSocket.
20 21 22 23 |
# File 'lib/ibm_appconfiguration_ruby_sdk/configurations/internal/websocket_client/driver_socket.rb', line 20 def initialize(tcp_socket, url) @tcp_socket = tcp_socket @url = url end |
Instance Attribute Details
#url ⇒ Object (readonly)
Returns the value of attribute url.
18 19 20 |
# File 'lib/ibm_appconfiguration_ruby_sdk/configurations/internal/websocket_client/driver_socket.rb', line 18 def url @url end |
Instance Method Details
#write(data) ⇒ Object
25 26 27 |
# File 'lib/ibm_appconfiguration_ruby_sdk/configurations/internal/websocket_client/driver_socket.rb', line 25 def write(data) @tcp_socket.write(data) end |