Class: IbmAppconfigurationRubySdk::EvaluationResult
- Inherits:
-
Struct
- Object
- Struct
- IbmAppconfigurationRubySdk::EvaluationResult
- Defined in:
- lib/ibm_appconfiguration_ruby_sdk/models/evaluation_result.rb
Overview
The result of evaluating a feature flag or remote property for a given entity.
Instance Attribute Summary collapse
-
#details ⇒ EvaluationDetails
readonly
Metadata about how the value was resolved.
-
#enabled ⇒ Boolean?
readonly
Whether the feature flag is enabled (nil for properties).
-
#value ⇒ Object
readonly
The resolved value of the feature flag or property.
Instance Attribute Details
#details ⇒ EvaluationDetails (readonly)
Returns Metadata about how the value was resolved.
53 |
# File 'lib/ibm_appconfiguration_ruby_sdk/models/evaluation_result.rb', line 53 EvaluationResult = Struct.new(:value, :enabled, :details, keyword_init: true) |
#enabled ⇒ Boolean? (readonly)
Returns Whether the feature flag is enabled (nil for properties).
53 |
# File 'lib/ibm_appconfiguration_ruby_sdk/models/evaluation_result.rb', line 53 EvaluationResult = Struct.new(:value, :enabled, :details, keyword_init: true) |
#value ⇒ Object (readonly)
Returns The resolved value of the feature flag or property.
53 |
# File 'lib/ibm_appconfiguration_ruby_sdk/models/evaluation_result.rb', line 53 EvaluationResult = Struct.new(:value, :enabled, :details, keyword_init: true) |